@@ -19,7 +19,7 @@
 #' @importFrom graphics Axis abline axTicks axis barplot box hist boxplot layout lcm lines locator par plot.new plot.window points
 #' @importFrom graphics polygon rect strheight strwidth text title
 #' @importFrom utils capture.output flush.console head
-#' @suggests dplyr
+#' @importFrom ranger ranger importance
 #' @useDynLib VIM
 NULL
 #' C-horizon of the Kola data with missing values
@@ -304,3 +304,5 @@ NULL
 
 
 
+setGeneric("plot")
+setGeneric("print")
\ No newline at end of file
